@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
     Copyright © 2018 Tobias Stein <stein.tobi@t-online.de>
 
@@ -61,6 +61,8 @@ namespace Magnum { namespace Trade {
 /**
 @brief TinyGltf importer plugin
 
+@m_keywords{GltfImporter GlbImporter}
+
 Imports glTF and binary glTF using the [TinyGLTF](https://github.com/syoyo/tinygltf)
 library.
 
@@ -75,6 +77,17 @@ of the `MagnumPlugins` package and link to the
 
 This plugin provides `GltfImporter` and `GlbImporter` plugins.
 
+@m_class{m-block m-success}
+
+@thirdparty This plugin makes use of the [TinyGLTF](https://github.com/syoyo/tinygltf)
+    library, licensed under @m_class{m-label m-success} **MIT**
+    ([license text](https://github.com/syoyo/tinygltf/blob/devel/LICENSE),
+    [choosealicense.com](https://choosealicense.com/licenses/mit/)).
+    It requires attribution for public use. TinyGLTF itself uses Niels
+    Lohmann's [json.hpp](https://github.com/nlohmann/json), licensed under
+    @m_class{m-label m-success} **MIT** as well
+    ([license text](https://github.com/nlohmann/json/blob/develop/LICENSE.MIT)).
+
 @section Trade-TinyGltfImporter-limitations Behavior and limitations
 
 The plugin supports @ref Feature::OpenData and @ref Feature::FileCallback
@@ -141,8 +154,10 @@ Import of skeleton, skin and morph data is not supported at the moment.
 
 @subsection Trade-TinyGltfImporter-limitations-lights Light import
 
--   Light properties besides light type are not imported.
--   Light intensity is not yet supported due to glTF extension draft state.
+-   Currently, the importer only supports the historical `KHR_lights_cmn`
+    extension, supported by the original Khronos Blender glTF Exporter.
+    -   Light properties besides light type are not imported.
+    -   Light intensity is not yet supported due to glTF extension draft state.
 
 @subsection Trade-TinyGltfImporter-limitations-meshes Mesh import
 
@@ -207,17 +222,74 @@ Import of skeleton, skin and morph data is not supported at the moment.
     -   The `snininessFactor` field is used for snininess, if present.
         Otherwise @cpp 1.0f @ce is used.
 
-@subsection Trade-TinyGltfImporter-limitations-textures Texture import
+@subsection Trade-TinyGltfImporter-limitations-textures Texture and image import
 
--   Texture type is always @ref Trade::TextureData::Type::Texture2D, as glTF
-    doesn't support anything else
--   Z coordinate of @ref Trade::TextureData::wrapping() is always
-    @ref SamplerWrapping::Repeat, as glTF doesn't support 3D textures
--   glTF leaves the defaults of sampler properties to the application, the
-    following defaults have been chosen for this importer:
-    -   Minification/magnification/mipmap filter: @ref SamplerFilter::Linear,
-        @ref SamplerMipmap::Linear
-    -   Wrapping (all axes): @ref SamplerWrapping::Repeat
+<ul>
+<li>Texture type is always @ref Trade::TextureData::Type::Texture2D, as glTF
+doesn't support anything else</li>
+<li>Z coordinate of @ref Trade::TextureData::wrapping() is always
+@ref SamplerWrapping::Repeat, as glTF doesn't support 3D textures</li>
+<li>
+@m_class{m-nopadb}
+
+glTF leaves the defaults of sampler properties to the application, the
+following defaults have been chosen for this importer:
+
+-   Minification/magnification/mipmap filter: @ref SamplerFilter::Linear,
+    @ref SamplerMipmap::Linear
+-   Wrapping (all axes): @ref SamplerWrapping::Repeat
+</li>
+<li>
+    The importer supports the non-standard `GOOGLE_texture_basis` extension
+    for referencing [Basis Universal](https://github.com/binomialLLC/basis_universal)
+    files, which then get loaded using @ref BasisImporter (or an equivalent
+    alias). The use is like this, [equivalently to Basis own glTF example](https://github.com/BinomialLLC/basis_universal/blob/1cae1d57266e2c95bc011b0bf1ccb9940988c184/webgl/gltf/assets/AgiHqSmall.gltf#L230-L240):
+
+    @code{.json}
+    {
+        ...
+        "textures": [
+            {
+                "extensions": {
+                    "GOOGLE_texture_basis": {
+                        "source": 0
+                    }
+                }
+            }
+        ],
+        "images": [
+            {
+                "mimeType": "image/x-basis",
+                "uri": "texture.basis"
+            }
+        ],
+        "extensionsUsed": [
+            "GOOGLE_texture_basis"
+        ],
+        "extensionsRequired": [
+            "GOOGLE_texture_basis"
+        ]
+    }
+    @endcode
+
+    The MIME type is not standard either and the importer doesn't check its
+    value. However, in case of embedded data URIs, the prefix *has to* be set
+    to `data:application/octet-stream` as TinyGLTF has a whitelist for data URI
+    detection and would treat the URI as a filename otherwise:
+
+    @code{.json}
+    {
+        ...
+        "images": [
+            {
+                "mimeType": "image/x-basis",
+                "uri": "data:application/octet-stream;base64,..."
+            }
+        ]
+    }
+    @endcode
+</li>
+</ul>
 
 @section Trade-TinyGltfImporter-configuration Plugin-specific config
 
@@ -308,49 +380,49 @@ class MAGNUM_TINYGLTFIMPORTER_EXPORT TinyGltfImporter: public AbstractImporter {
 
         MAGNUM_TINYGLTFIMPORTER_LOCAL Containers::Optional<CameraData> doCamera(UnsignedInt id) override;
         MAGNUM_TINYGLTFIMPORTER_LOCAL Int doCameraForName(const std::string& name) override;
-        MAGNUM_TINYGLTFIMPORTER_LOCAL std::string doCameraName(const UnsignedInt) override;
+        MAGNUM_TINYGLTFIMPORTER_LOCAL std::string doCameraName(UnsignedInt id) override;
         MAGNUM_TINYGLTFIMPORTER_LOCAL UnsignedInt doCameraCount() const override;
 
         MAGNUM_TINYGLTFIMPORTER_LOCAL Containers::Optional<LightData> doLight(UnsignedInt id) override;
         MAGNUM_TINYGLTFIMPORTER_LOCAL Int doLightForName(const std::string& name) override;
-        MAGNUM_TINYGLTFIMPORTER_LOCAL std::string doLightName(const UnsignedInt) override;
+        MAGNUM_TINYGLTFIMPORTER_LOCAL std::string doLightName(UnsignedInt id) override;
         MAGNUM_TINYGLTFIMPORTER_LOCAL UnsignedInt doLightCount() const override;
 
         MAGNUM_TINYGLTFIMPORTER_LOCAL Int doDefaultScene() override;
 
         MAGNUM_TINYGLTFIMPORTER_LOCAL Containers::Optional<SceneData> doScene(UnsignedInt id) override;
         MAGNUM_TINYGLTFIMPORTER_LOCAL Int doSceneForName(const std::string& name) override;
-        MAGNUM_TINYGLTFIMPORTER_LOCAL std::string doSceneName(const UnsignedInt) override;
+        MAGNUM_TINYGLTFIMPORTER_LOCAL std::string doSceneName(UnsignedInt id) override;
         MAGNUM_TINYGLTFIMPORTER_LOCAL UnsignedInt doSceneCount() const override;
 
         MAGNUM_TINYGLTFIMPORTER_LOCAL UnsignedInt doObject3DCount() const override;
         MAGNUM_TINYGLTFIMPORTER_LOCAL Int doObject3DForName(const std::string& name) override;
         MAGNUM_TINYGLTFIMPORTER_LOCAL std::string doObject3DName(UnsignedInt id) override;
-        MAGNUM_TINYGLTFIMPORTER_LOCAL std::unique_ptr<ObjectData3D> doObject3D(UnsignedInt id) override;
+        MAGNUM_TINYGLTFIMPORTER_LOCAL Containers::Pointer<ObjectData3D> doObject3D(UnsignedInt id) override;
 
         MAGNUM_TINYGLTFIMPORTER_LOCAL UnsignedInt doMesh3DCount() const override;
         MAGNUM_TINYGLTFIMPORTER_LOCAL Int doMesh3DForName(const std::string& name) override;
-        MAGNUM_TINYGLTFIMPORTER_LOCAL std::string doMesh3DName(const UnsignedInt) override;
-        MAGNUM_TINYGLTFIMPORTER_LOCAL Containers::Optional<MeshData3D> doMesh3D(const UnsignedInt id) override;
+        MAGNUM_TINYGLTFIMPORTER_LOCAL std::string doMesh3DName(UnsignedInt id) override;
+        MAGNUM_TINYGLTFIMPORTER_LOCAL Containers::Optional<MeshData3D> doMesh3D(UnsignedInt id) override;
 
         MAGNUM_TINYGLTFIMPORTER_LOCAL UnsignedInt doMaterialCount() const override;
         MAGNUM_TINYGLTFIMPORTER_LOCAL Int doMaterialForName(const std::string& name) override;
-        MAGNUM_TINYGLTFIMPORTER_LOCAL std::string doMaterialName(const UnsignedInt) override;
-        MAGNUM_TINYGLTFIMPORTER_LOCAL std::unique_ptr<AbstractMaterialData> doMaterial(const UnsignedInt id) override;
+        MAGNUM_TINYGLTFIMPORTER_LOCAL std::string doMaterialName(UnsignedInt id) override;
+        MAGNUM_TINYGLTFIMPORTER_LOCAL Containers::Pointer<AbstractMaterialData> doMaterial(UnsignedInt id) override;
 
         MAGNUM_TINYGLTFIMPORTER_LOCAL UnsignedInt doTextureCount() const override;
         MAGNUM_TINYGLTFIMPORTER_LOCAL Int doTextureForName(const std::string& name) override;
-        MAGNUM_TINYGLTFIMPORTER_LOCAL std::string doTextureName(const UnsignedInt) override;
-        MAGNUM_TINYGLTFIMPORTER_LOCAL Containers::Optional<TextureData> doTexture(const UnsignedInt id) override;
+        MAGNUM_TINYGLTFIMPORTER_LOCAL std::string doTextureName(UnsignedInt id) override;
+        MAGNUM_TINYGLTFIMPORTER_LOCAL Containers::Optional<TextureData> doTexture(UnsignedInt id) override;
 
         MAGNUM_TINYGLTFIMPORTER_LOCAL UnsignedInt doImage2DCount() const override;
         MAGNUM_TINYGLTFIMPORTER_LOCAL Int doImage2DForName(const std::string& name) override;
-        MAGNUM_TINYGLTFIMPORTER_LOCAL std::string doImage2DName(const UnsignedInt) override;
-        MAGNUM_TINYGLTFIMPORTER_LOCAL Containers::Optional<ImageData2D> doImage2D(const UnsignedInt id) override;
+        MAGNUM_TINYGLTFIMPORTER_LOCAL std::string doImage2DName(UnsignedInt id) override;
+        MAGNUM_TINYGLTFIMPORTER_LOCAL Containers::Optional<ImageData2D> doImage2D(UnsignedInt id) override;
 
         MAGNUM_TINYGLTFIMPORTER_LOCAL const void* doImporterState() const override;
 
-        std::unique_ptr<Document> _d;
+        Containers::Pointer<Document> _d;
 };
 
 }}
